@@ -33,9 +33,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #define LIBTORRENT_BUFFER_HPP
 
 #include <cstring>
+#include <limits> // for numeric_limits
 #include "libtorrent/invariant_check.hpp"
 #include "libtorrent/assert.hpp"
 #include <cstdlib> // malloc/free/realloc
+#include <boost/cstdint.hpp>
+#include <algorithm> // for std::swap
 
 namespace libtorrent {
 
@@ -44,14 +47,14 @@ class buffer
 public:
 	struct interval
 	{
-	   interval()
-		  : begin(0)
-		  , end(0)
+	interval()
+		: begin(0)
+		, end(0)
 		{}
 
-	   interval(char* b, char* e)
-		  : begin(b)
-		  , end(e)
+	interval(char* b, char* e)
+		: begin(b)
+		, end(e)
 		{}
 
 		char operator[](int index) const
@@ -59,7 +62,7 @@ public:
 			TORRENT_ASSERT(begin + index < end);
 			return begin[index];
 		}
-		  
+
 		int left() const
 		{
 			TORRENT_ASSERT(end >= begin);
@@ -73,14 +76,14 @@ public:
 
 	struct const_interval
 	{
-	   const_interval(interval const& i)
-		  : begin(i.begin)
-		  , end(i.end)
+	const_interval(interval const& i)
+		: begin(i.begin)
+		, end(i.end)
 		{}
 
-	   const_interval(char const* b, char const* e)
-		  : begin(b)
-		  , end(e)
+	const_interval(char const* b, char const* e)
+		: begin(b)
+		, end(e)
 		{}
 
 		char operator[](int index) const
@@ -108,16 +111,16 @@ public:
 
 	buffer(std::size_t n = 0)
 		: m_begin(0)
-		, m_end(0)
-		, m_last(0)
+		, m_size(0)
+		, m_capacity(0)
 	{
 		if (n) resize(n);
 	}
 
 	buffer(buffer const& b)
 		: m_begin(0)
-		, m_end(0)
-		, m_last(0)
+		, m_size(0)
+		, m_capacity(0)
 	{
 		if (b.size() == 0) return;
 		resize(b.size());
@@ -125,8 +128,26 @@ public:
 	}
 
 #if __cplusplus > 199711L
-	buffer(buffer&& b): m_begin(b.m_begin), m_end(b.m_end), m_last(b.m_last)
-	{ b.m_begin = b.m_end = b.m_last = NULL; }
+	buffer(buffer&& b)
+		: m_begin(b.m_begin)
+		, m_size(b.m_size)
+		, m_capacity(b.m_capacity)
+	{
+		b.m_begin = NULL;
+		b.m_size = b.m_capacity = 0;
+	}
+
+	buffer& operator=(buffer&& b)
+	{
+		if (&b == this) return *this;
+		std::free(m_begin);
+		m_begin = b.m_begin;
+		m_size = b.m_size;
+		m_capacity = b.m_capacity;
+		b.m_begin = NULL;
+		b.m_size = b.m_capacity = 0;
+		return *this;
+	}
 #endif
 
 	buffer& operator=(buffer const& b)
@@ -143,25 +164,28 @@ public:
 		std::free(m_begin);
 	}
 
-	buffer::interval data() { return interval(m_begin, m_end); }
-	buffer::const_interval data() const { return const_interval(m_begin, m_end); }
-	
+	buffer::interval data()
+	{ return interval(m_begin, m_begin + m_size); }
+	buffer::const_interval data() const
+	{ return const_interval(m_begin, m_begin + m_size); }
+
 	void resize(std::size_t n)
 	{
+		TORRENT_ASSERT(n < 0xffffffffu);
 		reserve(n);
-		m_end = m_begin + n;
+		m_size = boost::uint32_t(n);
 	}
 
 	void insert(char* point, char const* first, char const* last)
 	{
 		std::size_t p = point - m_begin;
-		if (point == m_end)
+		if (point == m_begin + m_size)
 		{
 			resize(size() + last - first);
 			std::memcpy(m_begin + p, first, last - first);
 			return;
 		}
-		
+
 		resize(size() + last - first);
 		std::memmove(m_begin + p + (last - first), m_begin + p, last - first);
 		std::memcpy(m_begin + p, first, last - first);
@@ -169,52 +193,58 @@ public:
 
 	void erase(char* b, char* e)
 	{
-		TORRENT_ASSERT(e <= m_end);
+		TORRENT_ASSERT(e <= m_begin + m_size);
 		TORRENT_ASSERT(b >= m_begin);
 		TORRENT_ASSERT(b <= e);
-	 	if (e == m_end)
+		if (e == m_begin + m_size)
 		{
 			resize(b - m_begin);
 			return;
 		}
-		std::memmove(b, e, m_end - e);
-		m_end = b + (m_end - e);
-	 }
+		std::memmove(b, e, m_begin + m_size - e);
+		TORRENT_ASSERT(e >= b);
+		TORRENT_ASSERT(e - b <= std::numeric_limits<boost::uint32_t>::max());
+		TORRENT_ASSERT(boost::uint32_t(e - b) <= m_size);
+		m_size -= e - b;
+	}
 
-	void clear() { m_end = m_begin; }
-	std::size_t size() const { return m_end - m_begin; }
-	std::size_t capacity() const { return m_last - m_begin; }
+	void clear() { m_size = 0; }
+	std::size_t size() const { return m_size; }
+	std::size_t capacity() const { return m_capacity; }
 	void reserve(std::size_t n)
 	{
 		if (n <= capacity()) return;
 		TORRENT_ASSERT(n > 0);
+		TORRENT_ASSERT(n < 0xffffffffu);
 
-		std::size_t s = size();
-		m_begin = (char*)std::realloc(m_begin, n);
-		m_end = m_begin + s;
-		m_last = m_begin + n;
+		char* tmp = static_cast<char*>(std::realloc(m_begin, n));
+#ifndef BOOST_NO_EXCEPTIONS
+		if (tmp == NULL) throw std::bad_alloc();
+#endif
+		m_begin = tmp;
+		m_capacity = boost::uint32_t(n);
 	}
 
-	bool empty() const { return m_begin == m_end; }
+	bool empty() const { return m_size == 0; }
 	char& operator[](std::size_t i) { TORRENT_ASSERT(i < size()); return m_begin[i]; }
 	char const& operator[](std::size_t i) const { TORRENT_ASSERT(i < size()); return m_begin[i]; }
 
 	char* begin() { return m_begin; }
 	char const* begin() const { return m_begin; }
-	char* end() { return m_end; }
-	char const* end() const { return m_end; }
+	char* end() { return m_begin + m_size; }
+	char const* end() const { return m_begin + m_size; }
 
 	void swap(buffer& b)
 	{
 		using std::swap;
 		swap(m_begin, b.m_begin);
-		swap(m_end, b.m_end);
-		swap(m_last, b.m_last);
+		swap(m_size, b.m_size);
+		swap(m_capacity, b.m_capacity);
 	}
 private:
-	char* m_begin; // first
-	char* m_end; // one passed end of size
-	char* m_last; // one passed end of allocation
+	char* m_begin;
+	boost::uint32_t m_size;
+	boost::uint32_t m_capacity;
 };
 
 
